#!/usr/bin/eperl
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#

package VHOptions;

require VHConst;
require VHCreateFS;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;


#
# The add functions need $::user and $::group to be defined as "local"
# when they are called.
#

# Adding analog to a domain
sub AddAnalog() {
    my ($record)=@_;
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Analog"));
}

#Removing analog from a domain
sub RemoveAnalog() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Analog"));
}

# Adding Web to a domain
sub AddWeb() {
    my ($record)=@_;
    symlink("/home/virtual/$record->{User}/home/httpd/html" , "/home/httpd/html/$record->{User}");
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Web"));
}

#Removing Web from a domain
sub RemoveWeb() {
    my ($record)=@_;

    unlink("/home/httpd/html/$record->{User}");
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Web"));
}

# Adding CGI to a domain
sub AddCGI() {
    my ($record)=@_;
    my(@directories)=(["/home/httpd/$record->{ScriptAlias}","$::user","$::group",0755],);
    my(%hardlinks)=();
    my(@newfiles)=();
    return &VHCreateFS::ActualCreateFilesystem(\@directories,\%hardlinks,\@newfiles);
}

# Removing CGI from a domain
sub RemoveCGI() {
    my ($record)=@_;
    my(@directories)=(["/home/httpd/$record->{ScriptAlias}","$::user","$::group",0755],);
    my(%hardlinks)=();
    my(@newfiles)=();
    return &VHCreateFS::ActualDeleteFilesystem(\@directories,\%hardlinks,\@newfiles);
}

# Changing ScriptAlias of a domain
sub ChangeScriptAlias() {
    my ($record, $new_record) = @_;
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    system("/bin/mv $vroot/home/httpd/$record->{ScriptAlias} $vroot/home/httpd/$new_record->{ScriptAlias} >/dev/null 2>/dev/null");
}

# Adding SSL to a domain
sub AddSSL() {
    my ($record)=@_;
 
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("openssl"));    
}

#Removing SSL from a domain
sub RemoveSSL() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("openssl"));
}

# Adding sendmail to a domain
sub AddSendmail() {
    my ($record)=@_;
    my ($error_message);
    if ($error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Sendmail"))) {
	return $error_message;
    }
    my ($prefix)="$VHConst::VIRTUALFS/$::user";

    #TODO:???:what is this? Inconsistent??
    system ("cp -f /etc/ld.so.conf $prefix/etc/ld.so.conf");
    system ("cd $prefix; /usr/sbin/chroot . /sbin/ldconfig >/dev/null 2>/dev/null");
    {
	open (SMCF,">>$prefix/etc/mail/sendmail.cf")
	  or $cerr += $E_UNABLEAPPENDFILE 
      and $cerr += {file => "$prefix/etc/mail/sendmail.cf" }
      and return "Could not open $prefix/etc/mail/sendmail.cf for appending.\n";
	my $domain = $record->{Domain};
	my @doms = split (/\./, $domain);
	my $host = $doms[0];
	shift @doms;
	my $dom = join (".",@doms);
	#append magic lines
	print SMCF "\nDw$host\nDm$dom\nDj$domain\nDk$domain\n";
	close (SMCF);
	open (SMCW,">>$prefix/etc/sendmail.cw")
	  or $cerr += $E_UNABLEAPPENDFILE 
      and $cerr += {file => "$prefix/etc/sendmail.cw" }
      and return "Could not open $prefix/etc/sendmail.cw for appending.\n";
	#append the domain
	print SMCW "$domain\n";
	close (SMCW);
    }
    #setting up aliases
    system ("cd $prefix; /usr/sbin/chroot . /usr/bin/newaliases >/dev/null 2>/dev/null");
    return "";
}

#Removing sendmail from a domain
sub RemoveSendmail() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Sendmail"));
}

# Adding IMAP to a domain
sub AddIMAP() {
    my ($record)=@_;
    my ($prefix)="$VHConst::VIRTUALFS/$::user";
    my ($error_message) = "";
    if ($error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("IMAP"))) {
	return $error_message;
    }
    system ("cp -f /etc/ld.so.conf $prefix/etc/ld.so.conf");
    system ("cd $prefix; /usr/sbin/chroot . /sbin/ldconfig >/dev/null 2>/dev/null");
    return "";
}

#Removing IMAP from a domain
sub RemoveIMAP() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("IMAP"));
}

# Adding MySQL database for domain
# If database exists, then we want to set the correct
# access permissions and preserve existing data.  If database
# does not exist, then we create the database and set correct
# access permissions.
sub AddMySQL() {
    require VHMySQL;
    my ($record) = @_ ;
    my ($result) ;
    
    $result = &VHMySQL::DatabaseExists($record->{User}) ;
    chomp($result) ;
    if ( $result eq "Y" ) {
        return &VHMySQL::GrantDatabase ($record->{User}) ;
    }
    else {
        return &VHMySQL::CreateDatabase ($record->{User}) ;
    }
}

# Removing MySQL database from domain
# This does not actually remove the database.  It disables the
# database from access via its owner.  Database is removed
# in DeleteVirtDomain
sub RemoveMySQL() {
    require VHMySQL;

    my ($record) = @_ ;
    return &VHMySQL::RevokeDatabase ($record->{User}) ;
}

# Adding Miva Merchant to a domain
sub AddMivaMerchant() {
    my ($record)=@_;
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("MivaMerchant"));
}

# Removing Miva Merchant from a domain
sub RemoveMivaMerchant() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("MivaMerchant"));
}

# Adding vacation to a domain
sub AddVacation() {
    my ($record)=@_;
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Vacation"));
}

#Removing vacation from a domain
sub RemoveVacation() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Vacation"));
}

# Adding majordomo to a domain
sub AddMajordomo() {
    my ($record)=@_;
    my ($error_message);
    if ($error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Majordomo"))) {
	return $error_message;
    }
    my($prefix)="$VHConst::VIRTUALFS/$::user";
    # special case for majordomo,
    # as we put in our special file, 'wrapper' file into the
    # virt domain filesystem
    symlink("/usr/lib/majordomo/wrapper", "$prefix/etc/smrsh/wrapper");
    {
	open (ALIAS,">>$prefix/etc/aliases")
	  or $cerr += $E_UNABLEAPPENDFILE
      and $cerr += { file => "$prefix/etc/aliases" } 
      and return "Could not open $prefix/etc/aliases for appending.\n";
	print ALIAS "majordomo:\t\"|/usr/lib/majordomo/wrapper majordomo\"\n";
	# we also need to restore the aliases for any old mailing lists.
	if (opendir (LISTS,"$prefix/var/lib/majordomo/lists")) {
	    my (@lists) = readdir(LISTS);
	    closedir(LISTS);
	    @lists = grep {$_ !~ /\.config$/} @lists;
	    @lists = grep {$_ !~ /\.info$/} @lists;
	    my ($list);
	    foreach $list (@lists) {
		next if ($list eq "." or $list eq "..");
		print ALIAS "$list: :include:/var/lib/majordomo/lists/$list\n";
		print ALIAS "$list-list: :include:/var/lib/majordomo/lists/$list\n";
		print ALIAS "$list-request: \"|/usr/lib/majordomo/wrapper majordomo -l $list\"\n";
	    }
	}
	close (ALIAS);
    }
    #call newaliases
    system ("cd $prefix; /usr/sbin/chroot . /usr/bin/newaliases >/dev/null 2>/dev/null");
    {
	open (LOGRMAJ,">$prefix/etc/logrotate.d/majordomo")
	  or $cerr += $E_UNABLEWRITEFILE
      and $cerr += { file => "$prefix/etc/logrotate.d/majordomo" } 
      and return "Could not open $prefix/etc/logrotate.d/majordomo for writing.\n";
	print LOGRMAJ <<"EOF";
$VHConst::VIRTUALFS/$::user/var/lib/majordomo/log {
    missingok
    weekly
    compress
    delaycompress
    olddir $VHConst::VIRTUALFS/$::user/var/lib/majordomo/OLDLOGS
    rotate 52
    create 640 $::user $::user
}
EOF
        close (LOGRMAJ);
    }
    {
            my ($uid);
            $uid = getpwnam($record->{User});
	open (PASSWD,">>$prefix/etc/passwd")
	  or $cerr += $E_UNABLEAPPENDFILE
      and $cerr += { file => "$prefix/etc/passwd" }
      and return "Could not open $prefix/etc/passwd for appending.\n";
	print PASSWD "majordomo:x:$uid:$uid:Majordomo List Manager:/usr/lib/majordomo:/dev/null\n";
	close (PASSWD);
    }
    {
            my ($uid);
            $uid = getpwnam($record->{User});
	open (GROUP,">>$prefix/etc/group")
	  or  $cerr += $E_UNABLEAPPENDFILE
      and  $cerr += { file => "$prefix/etc/group" }
      and return "Could not open $prefix/etc/group for appending.\n";
	print GROUP "majordomo:x:$uid:mail\n";
	close (GROUP);
    }
    return "";
}

#Removing majordomo from a domain
sub RemoveMajordomo() {
    my ($record)=@_;
    my($prefix)="$VHConst::VIRTUALFS/$::user";
    unlink("$VHConst::VIRTUALFS/$record->{User}/etc/smrsh/wrapper");
    #removing the majordomo entry from /etc/aliases
    {
	open (ALIAS,"$prefix/etc/aliases")
	  or $cerr += $E_UNABLEAPPENDFILE
      and $cerr += { file => "$prefix/etc/aliases" }
      and print STDERR "Could not open $prefix/etc/aliases for reading.\n";
	my (@alias) = <ALIAS>;
	close (ALIAS);
	@alias = grep {$_ !~ /^majordomo:/} @alias;
	@alias = grep {$_ !~ /:include:\/var\/lib\/majordomo\/lists\//} @alias;
	@alias = grep {$_ !~ /\/usr\/lib\/majordomo\/wrapper/} @alias;
	open (ALIAS,">$prefix/etc/aliases")
	  or $cerr += $E_UNABLEWRITEFILE
      and $cerr += { file => "$prefix/etc/aliases" }
      and print STDERR "Could not open $prefix/etc/aliases for writing.\n";
	print ALIAS @alias;
	close (ALIAS);
    }
    #call newaliases
    system ("cd $prefix; /usr/sbin/chroot . /usr/bin/newaliases >/dev/null 2>/dev/null");
    #removing the passwd and group entries
    {
	open (PASSWD,"$prefix/etc/passwd")
	  or $cerr += $E_UNABLEREADFILE
      and $cerr += { file => "$prefix/etc/passwd" }
      and print STDERR "Could not open $prefix/etc/passwd for reading.\n";
	my (@passwd) = <PASSWD>;
	close (PASSWD);
	@passwd = grep {$_ !~ /^majordomo:/} @passwd;
	open (PASSWD,">$prefix/etc/passwd")
	  or $cerr += $E_UNABLEWRITEFILE
      and $cerr += { file => "$prefix/etc/passwd" }
      and print STDERR "Could not open $prefix/etc/passwd for writing.\n";
	print PASSWD @passwd;
	close (PASSWD);
    }
    {
	open (GROUP,"$prefix/etc/group")
	  or $cerr += $E_UNABLEREADFILE
      and $cerr += { file => "$prefix/etc/group" }
      and print STDERR "Could not open $prefix/etc/group for reading.\n";
	my (@group) = <GROUP>;
	close (GROUP);
	@group = grep {$_ !~ /^majordomo:/} @group;
	open (GROUP,">$prefix/etc/group")
	  or $cerr += $E_UNABLEWRITEFILE
      and $cerr += { file => "$prefix/etc/group" }
      and print STDERR "Could not open $prefix/etc/group for writing.\n";
	print GROUP @group;
	close (GROUP);
    }
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Majordomo"));
}

# Adding telnet to a domain
sub AddTelnet() {
    my ($record)=@_;

    my ($error_message);

    # for NB domains, we dont need to mount. /dev/pts is needed by in.telnetd, 
    # and for NB domains it uses the one on the box
    # we dont need the file system either
    if ($record->{DomainType} eq "NameBased") { return "";}

    $error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("Telnet"));
    if ($error_message) {return $error_message;}

    system("/bin/mount -t devpts none $VHConst::VIRTUALFS/$::user/dev/pts >/dev/null 2>/dev/null"); 
    if (($? >> 8) != 0){
    $cerr += $E_UNABLEMOUNT and $cerr += { what => "/dev/pts" } ;
	return "Cannot mount /dev/pts .\n";
    }
    return ""; 
}

#Removing telnet from a domain
sub RemoveTelnet() {
    my ($record)=@_;

    #kill the connections, umount /dev/pts (for IP based), etc
    print STDERR &VHCommon::PrepareDisableTelnet($record);
    # if name based, no file system is needed for the domain
    if ($record->{DomainType} eq "NameBased") { return "";}
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("Telnet"));
}

# Adding FTP to a domain
sub AddFTP() {
    my ($record)=@_;
    if ($record->{DomainType} eq "NameBased") {return "";}
    my ($error_message);
    $error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("FTP"));
    if ($error_message) {return $error_message;}
    my ($prefix)="$VHConst::VIRTUALFS/$::user";
    system ("/bin/touch $prefix/var/log/xferlog");
    my (@pwent) = getpwnam($record->{User});
    my ($uid,$gid) = ($pwent[2],$pwent[3]);
    chown ($uid,$gid,"$prefix/var/log/xferlog") or
    $cerr += $E_UNABLECHOWNFILEUSR and
    $cerr += { file => "$prefix/var/log/xferlog", whom => "$uid $gid", perror => $! } and
      return "Could not change ownership of $prefix/var/log/xferlog to $uid $gid: $! .\n";
    chmod (0644,"$prefix/var/log/xferlog") or 
    $cerr += $E_UNABLECHMODFILE and 
    $cerr += { file => "$prefix/var/log/xferlog", perror => $! } and
      return "Could not modify permissions of $prefix/var/log/xferlog: $! .\n";
    {
	open (PASSWD,">>$prefix/etc/passwd")
	  or $cerr += $E_UNABLEAPPENDFILE 
      and $cerr += { file => "$prefix/etc/passwd" } 
      and return "Could not open $prefix/etc/passwd for appending.\n";
	print PASSWD "ftp:x:$uid:$uid:FTP user:/home/ftp:/bin/ftponly\n";
	close (PASSWD);
    }
    {
        open (CONF,">>$prefix/etc/proftpd.conf")
          or $cerr += $E_UNABLEAPPENDFILE 
          and $cerr += { file => "$prefix/etc/proftpd.conf" } 
          and return "Could not append to $prefix/etc/proftpd.conf .\n";
        print CONF <<"EOF";
# Set the user and group that the server normally runs at.
User                           $::user
Group                          $::group
EOF
        if ($record->{AnonFtp}) {
            print CONF &VHAppConf::AnonymousFtpConfEntry();
        }
        close (CONF);
    }
    {
      open (LOGRFTP,">$prefix/etc/logrotate.d/proftpd")
        or $cerr += $E_UNABLEWRITEFILE and
        $cerr += { file => "$prefix/etc/logrotate.d/proftpd" } and
        return "Could not open $prefix/etc/logrotate.d/proftpd for writing.\n";
      print LOGRFTP <<"EOF";
$VHConst::VIRTUALFS/$::user/var/log/xferlog {
    missingok
    postrotate
    endscript
}
EOF
      close (LOGRFTP);
    }
    $error_message = &VHAppConf::ChangeFtpServerName($record->{FtpServerName});
    return $error_message;
}

#Removing FTP from a domain
sub RemoveFTP() {
    my ($record)=@_;
    if ($record->{DomainType} eq "NameBased") {return "";}
    my ($prefix)="$VHConst::VIRTUALFS/$::user";
    if ( -z "$prefix/var/log/xferlog" ) {
	unlink ("$prefix/var/log/xferlog") or
    $cerr += $E_USERDATANOTDEL and
    $cerr += { data => "$prefix/var/log/xferlog" } and
	  print STDERR "User data has not been deleted: $prefix/var/log/xferlog .\n";
    }
    {
	open (PASSWD,"$prefix/etc/passwd")
	  or $cerr += $E_UNABLEREADFILE
      and $cerr += { file => "$prefix/etc/passwd" } 
      and print STDERR "Could not open $prefix/etc/passwd for reading.\n";
	my (@passwd) = <PASSWD>;
	close (PASSWD);
	@passwd = grep {$_ !~ /^ftp:/} @passwd;
	open (PASSWD,">$prefix/etc/passwd")
	  or $cerr += $E_UNABLEWRITEFILE
      and $cerr += { file => "$prefix/etc/passwd" } 
      and print STDERR "Could not open $prefix/etc/passwd for writing.\n";
	print PASSWD @passwd;
	close (PASSWD);
    }
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("FTP"));
}

# Adding AnonFtp to a domain
sub AddAnonFtp() {
    my ($record)=@_;
    my ($error_message);
    return &VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("AnonFtp"));
}

#Removing AnonFtp from a domain
sub RemoveAnonFtp() {
    my ($record)=@_;
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("AnonFtp"));
}

# Adding web logs to a domain
sub AddWebLogs() {
    my ($record)=@_;
    my ($error_message);
    $error_message=&VHCreateFS::ActualCreateFilesystem(&VHCreateFS::GetFileList("WebLogs"));
    if ($error_message) {return $error_message;}
    my ($prefix)="$VHConst::VIRTUALFS/$::user";
    my (@pwent) = getpwnam($record->{User});
    my ($uid,$gid) = ($pwent[2],$pwent[3]);
    system ("/bin/touch $prefix/var/log/httpd/access_log");
    chown ($uid,$gid,"$prefix/var/log/httpd/access_log") or
    $cerr += $E_UNABLECHOWNFILEUSR and
    $cerr += {file => "$prefix/var/log/httpd/access_log", whom => "$uid $gid", perror => $! } and 
      $error_message .= "Could not change ownership of $prefix/var/log/httpd/access_log to $uid $gid: $! .\n";
    chmod (0644,"$prefix/var/log/httpd/access_log") or 
    $cerr += $E_UNABLECHMODFILE and
    $cerr += { file => "$prefix/var/log/httpd/access_log" , perror => $! } and
      $error_message .= "Could not modify permissions of $prefix/var/log/httpd/access_log: $! .\n";
    system ("/bin/touch $prefix/var/log/httpd/error_log");
    chown ($uid,$gid,"$prefix/var/log/httpd/error_log") or
      $error_message .= "Could not change ownership of $prefix/var/log/httpd/error_log to $uid $gid: $! .\n";
    chmod (0644,"$prefix/var/log/httpd/error_log") or 
    $cerr += $E_UNABLECHMODFILE and
    $cerr += { file => "$prefix/var/log/httpd/error_log" , perror => $! } and
      $error_message .= "Could not modify permissions of $prefix/var/log/httpd/error_log: $! .\n";
    {
      open (LOGRAP,">$prefix/etc/logrotate.d/apache")
	or $cerr += $E_UNABLEWRITEFILE
    and $cerr += { file => "$prefix/etc/logrotate.d/apache" } 
    and return "Could not open $prefix/etc/logrotate.d/apache for writing.\n";
      print LOGRAP <<"EOF";
$VHConst::VIRTUALFS/$::user/var/log/httpd/access_log {
    missingok
    postrotate
        /usr/bin/killall -HUP httpd
    endscript
}

$VHConst::VIRTUALFS/$::user/var/log/httpd/error_log {
    missingok
    postrotate
        /usr/bin/killall -HUP httpd
    endscript
}
EOF
      close (LOGRAP);
    }
    return $error_message;
}

#Removing web logs from a domain
sub RemoveWebLogs() {
    my ($record)=@_;
    my ($prefix)="$VHConst::VIRTUALFS/$::user";
    if ( -z "$prefix/var/log/httpd/access_log") {
	unlink ("$prefix/var/log/httpd/access_log") or
    $cerr += $E_USERDATANOTDEL and $cerr += {data => "$prefix/var/log/httpd/access_log" } and 
	  print STDERR "User data has not been deleted: $prefix/var/log/httpd/access_log .\n";
    }
    if ( -z "$prefix/var/log/httpd/error_log") {
	unlink ("$prefix/var/log/httpd/error_log") or
    $cerr += $E_USERDATANOTDEL and $cerr += {data => "$prefix/var/log/httpd/error_log" } and 
	  print STDERR "User data has not been deleted: $prefix/var/log/httpd/error_log .\n";
    }
    return &VHCreateFS::ActualDeleteFilesystem(&VHCreateFS::GetFileList("WebLogs"));
}

# Adds back NB Users mail boxes to Server so they can get mail (imap/pop)
sub AddNBMailBoxesToServer() {
    my ($record) = @_;
    # Get virtual users in this domain
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    my (@lines);

    unless(open (PASSWD, "$vroot/etc/passwd")) {
        $cerr += $E_UNABLEREADVIRTPWDFILE and 
        $cerr += { vroot => $vroot } ;
        return "Cannot read virtual password file $vroot/etc/passwd .\n";
    }
    @lines = <PASSWD>;
    close (PASSWD);
    my ($error_message, $local_error) = "";
    if ($error_message = &VHConfiguration::ReadUIDConfFile()) {
        return $error_message;
    }
    my ($line, @fields, $uid, $gid, $fullname, $user);

    foreach $line (@lines) {
        chop ($line);
        @fields = split(/:/,$line);
        $uid = $fields[2];
        $gid = $fields[3];
	#The condition below means: if the user is *not* a site administrator
        if ($uid>$VHConfiguration::MinVirtUid and $uid<$VHConfiguration::MaxVirtUid) {
            $fullname = $fields[4];
            $user = $fields [0];
	    system ( "echo \'processing mail box of user $user\' >> /tmp/junk");
            # Add user to server
            if ($error_message = &VHCommon::AddNBMailBox($user,$uid, $gid,$record)) {
                return $error_message;
            }
        }
    }
    return "";
}

# This function must *not* be used for enabling/disabling imap, 
# since NB users on the server are now used for ftp and telnet also
# Deletes NB Users from Server. Called in DeleteVirtDomain only 
sub DeleteNBUsersFromServer() {
    my ($record) = @_;

    # Get virtual users in this domain
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    my (@lines);
    unless(open (PASSWD, "$vroot/etc/passwd")) {
        $cerr += $E_UNABLEREADVIRTPWDFILE and 
        $cerr += { vroot => $vroot } ;
        return "Cannot read virtual password file $vroot/etc/passwd .\n";
    }
    @lines = <PASSWD>;
    close (PASSWD);
    my ($error_message) = "";
    if ($error_message = &VHConfiguration::ReadUIDConfFile()) {
        return $error_message;
    } else {
        #This line should not be here.Somehow this function returns "" and we
        #get undef.
        $error_message = ""
    }
    my ($line, @fields, $uid, $fullname, $user, $org, $dst);
    foreach $line (@lines) {
        chop ($line);
        @fields = split(/:/,$line);
        $uid = $fields[2];
	# The condition below means: if the user is *not* a site administrator
        if ($uid>$VHConfiguration::MinVirtUid and $uid<$VHConfiguration::MaxVirtUid) {
            $user = $fields [0];
            # remove the user mail boxes from server
            $error_message.=&VHCommon::DelNBVirtUser($user,$record);
        }
    }
    return $error_message;
}

# Deletes NB Users mail boxes from Server so they won't get mail (imap/pop)
sub DeleteNBMailBoxesFromServer() {
    my ($record) = @_;

    # Get virtual users in this domain
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    my (@lines);
    unless(open (PASSWD, "$vroot/etc/passwd")) {
        $cerr += $E_UNABLEREADVIRTPWDFILE and 
        $cerr += { vroot => $vroot } ;
        return "Cannot read virtual password file $vroot/etc/passwd .\n";
    }
    @lines = <PASSWD>;
    close (PASSWD);
    my ($error_message) = "";
    if ($error_message = &VHConfiguration::ReadUIDConfFile()) {
        return $error_message;
    } else {
        #This line should not be here.Somehow this function returns "" and we
        #get undef.
        $error_message = ""
    }
    my ($line, @fields, $uid, $fullname, $user, $org, $dst);
    foreach $line (@lines) {
        chop ($line);
        @fields = split(/:/,$line);
        $uid = $fields[2];
	# The condition below means: if the user is *not* a site administrator
        if ($uid>$VHConfiguration::MinVirtUid and $uid<$VHConfiguration::MaxVirtUid) {
            $user = $fields [0];
            # remove the user mail boxes from server
            $error_message.=&VHCommon::DelNBMailBox($user,$record);
        }
    }
    return $error_message;
}


