# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# This module handles the quota related functions.
#

package VHQuota;

BEGIN {}

END {}

require VHConst;
require VHCommon;
require Quota;
require Filesys::Statvfs;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

my ($bs) = 1024;

sub getBlockSize() {
  my (@diskdata)=Filesys::Statvfs::statvfs($VHConst::VIRTUALFS);
  my ($bsize);
  if (not defined($diskdata[0])) {
  $cerr += $E_UNABLEESTIMATEDU and $cerr += { perror => $! } ;
  $cerr += $E_ASSUMEONEK ;
    print STDERR "Cannot estimate disk usage: $!\nAssuming block size of 1k\n";
    $bsize=1024;
  } else {
    $bsize=$diskdata[0];
  }
  return $bsize
}

#
# This function acually sets a (user or group) quota. It emits a
# warning if the quota exceeds the available space (+ space already
# used) or the size of the partition.
#
sub PerformSetQuota() {
    my($uid,$quotamegs,$uidgidflag)=@_;
    my($dev)=Quota::getqcarg($VHConst::VIRTUALFS);
    if (not defined $dev or not $dev) {
    $cerr += $E_UNSUPPORTEDQUOTA ;
	return "Quotas not supported (for example: quotas are not supported on NFS servers)\n";
    }
    #convert from megs to blocks
    my (@diskdata)=Filesys::Statvfs::statvfs($VHConst::VIRTUALFS);
    my ($bsize) = $bs;
    my($quota)=$quotamegs*1048576/$bsize;
    #Actually set the quota
    if(Quota::setqlim($dev,$uid,$quota,$quota,0,0,0,$uidgidflag)!=0) {
    $cerr += $E_QUOTASETERR and $cerr += { err => (Quota::strerr()) };
	return "Errors while setting quota. ".(Quota::strerr())."\n";
    }
    #Perform some checks
    if (defined($diskdata[0])) {
        my ($block_size) = &getBlockSize();
	my($bavail,$blocks)=(($diskdata[4] * $block_size) / $bs,($diskdata[2] * $block_size) / $bs);
	if ($quota > $blocks) {
    $cerr += $E_QUOTAMORETHANSPACE and $cerr += { quota => $quotamegs, space => (&VHQuota::formatquota($blocks)) };
	    print STDERR "The allocated quota ($quotamegs) is more than the total space: ".(&VHQuota::formatquota($blocks))."\n";
	} else {
	    my($bc,@junk)=Quota::query($dev,$uid,$uidgidflag); 
	    unless(defined ($bc) and ($bc > 0 or $bc eq "0")) {
        $cerr += $E_QUOTACHKERR and $cerr += { err => (Quota::strerr()) };
		print STDERR "Errors checking quota. ".(Quota::strerr())."\n";
		return "";
	    }
	    if ($quota > $bavail+$bc) {
        $cerr += $E_QUOTAMORETHANSUM and
        $cerr += { allocquota => $quotamegs,
            free => (&VHQuota::formatquota($bavail)),
            quota => (&VHQuota::formatquota($bc)) };
		print STDERR "The allocated quota ($quotamegs) is more than the sum of free space (".(&VHQuota::formatquota($bavail)).") and space already counted against the quota (".(&VHQuota::formatquota($bc)).").\n";
	    }
	}
    }
    return "";
}

sub SetDomainQuota() {
    my($record)=@_;
    my(@pwent)=getpwnam($record->{User});
    unless(@pwent){
    $cerr += $E_UNABLEGETUSERINFO and $cerr += { user => $record->{User} };
	return "Could not get user information for $record->{User} .\n";
    }
    return &VHQuota::PerformSetQuota($pwent[3],$record->{Quota},1);
}

sub SetQuotaVirtUser() {
    my($user,$quota,$record) = @_;
    my($error_message);
    my($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    unless (open(PASSWD,"$vroot/etc/passwd")) {
    $cerr += $E_UNABLEREADPWDFILE and $cerr += { vroot => $vroot };
    $cerr += $E_USERADDERRORS ;
	return "Cannot read password file $vroot/etc/passwd .\nErrors during useradd.\n";
    }
    my(@lines)=<PASSWD>;
    close (PASSWD);
    @lines = grep {$_ =~ /^$user:/} @lines;
    unless ($#lines == 0) {
    $cerr += $E_USERNOTADDED ;
	return "User not properly added to password file.\n";
    }
    my(@pwent)=split(/:/,$lines[0]);
    #We have to check though that this is actually a virtual user
    &VHConfiguration::ReadUIDConfFile();
    unless ($pwent[2] >= $VHConfiguration::MinVirtUid and
	    $pwent[2] <= $VHConfiguration::MaxVirtUid) {
        $cerr += $E_UIDNOTVIRTUAL and
        $cerr += { user => $user, uid => $pwent[3] };
	return "UID for user $user ($pwent[3]) is not a virtual UID.\n";
    }
    #Check if the group has enough space available
    if ($error_message = &VHQuota::GetDomainUsage($record)) {
        return $error_message;
    }
    if ($quota > $record->{Quota}-$record->{UsedQuota}) {
    $cerr += $E_USERQGTFREESPACE and
    $cerr += { uq => $quota, sq => $record->{Quota}, used => $record->{UsedQuota} };
        print STDERR "The user quota ($quota) exceeds the space left for the whole site (site quota is $record->{Quota}, site uses $record->{UsedQuota})\n";
    }
    #Check if the group has enough space available
    if ($error_message = &VHQuota::GetDomainAllocUsage($record, $pwent[2])) {
        return $error_message;
    }
    if ($quota > $record->{Quota}-$record->{AllocQuota}) {
        $cerr += $E_USERQGTSITEQ and
        $cerr += { uq => $quota, sq => $record->{Quota}, aq => $record->{AllocQuota} };
        print STDERR "The user quota ($quota) exceeds the space already allocated for the whole site (site quota is $record->{Quota}, site already allocated to users $record->{AllocQuota})\n";
    }
    #Now we actually set the quota
    return &VHQuota::PerformSetQuota($pwent[2],$quota,0);
}

sub GetDomainUsage() {
    my($record)=@_;

    my(@pwent)=getpwnam($record->{User});
    unless(@pwent){
    $cerr += $E_UNABLEGETUSERINFO and $cerr += { user => $record->{User} };
	return "Could not get user information for $record->{User} .\n";
    }
    my($dev)=Quota::getqcarg($VHConst::VIRTUALFS);
    if (not defined $dev or not $dev) {
    $cerr += $E_UNSUPPORTEDQUOTA ;
	return "Quotas not supported (for example: quotas are not supported on NFS servers)\n";
    }
    my($bc,$bs,$bh,$bt, $ic,$is,$ih,$it)=Quota::query($dev,$pwent[3],1); 
    unless(defined ($bc) and ($bc > 0 or $bc eq "0")) {
    $cerr += $E_READQINFO and $cerr += { err => (Quota::strerr()) };
	return "Errors while reading quota information. ".(Quota::strerr())."\n";
    }
    $record->{UsedQuota}= &formatquota($bc); disabled quota
    return "";
}

sub GetDomainAllocUsage() {
    my($record,$uid)=@_;
    my($allocquota)=0;
    my($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    my($dev)=Quota::getqcarg($VHConst::VIRTUALFS);
    if (not defined $dev or not $dev) {
        $cerr += $E_UNSUPPORTEDQUOTA ;
        return "Quotas not supported (for example: quotas are not supported on NFS servers)\n";
    }
    unless (open(PASSWD,"$vroot/etc/passwd")) {
        $cerr += $E_UNABLEGETUSERDOMAININFO and $cerr += { domain => $record->{Domain} };
        return "Cannot get user information for domain $record->{Domain}\n";
    }
    my(@lines)=<PASSWD>;
    my($line)="";
    my(@pwent,$bc,$bs,$bh,$bt,$ic,$is,$ih,$it);
    close (PASSWD);
    foreach $line (@lines) {
        @pwent=split(/:/,$line);
        # don't add a users allocation if this is the user we're setting
        if ($uid != $pwent[2]) {
            ($bc,$bs,$bh,$bt,$ic,$is,$ih,$it)=Quota::query($dev,$pwent[2],0);
            unless(defined ($bh) and ($bh > 0 or $bh eq "0")) {
                $cerr += $E_READQINFO and $cerr += { err => (Quota::strerr()) };
                return "Errors while reading quota information. ".(Quota::strerr())."\n";
            }
            $allocquota = $allocquota + &formatquota($bh);
        }
    }
    $record->{AllocQuota} = $allocquota;
    return "";
}

# On failure it returns an error message, on success, it returns a
# pointer to a scalar containnig the disk usage of the virtual user
# (in megabytes).
sub GetUsageVirtUser() {
    my($user,$record) = @_;
    #We first get the uid of the virtual user
    my($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    unless (open(PASSWD,"$vroot/etc/passwd")) {
    $cerr += $E_UNABLEREADPWDFILE and $cerr += { vroot => $vroot };
    $cerr += $E_USERADDERRORS ;
	return "Cannot read password file $vroot/etc/passwd .\nErrors during useradd.\n";
    }
    my(@lines)=<PASSWD>;
    close (PASSWD);
    @lines = grep {$_ =~ /^$user:/} @lines;
    unless ($#lines == 0) {
    $cerr += $E_USERNOTADDED ;
	return "User not properly added to password file.\n";
    }
    my(@pwent)=split(/:/,$lines[0]);
    #Now we actually get the usage information from the quota system
    my($dev)=Quota::getqcarg($VHConst::VIRTUALFS);
    if (not defined $dev or not $dev) {
    $cerr += $E_UNSUPPORTEDQUOTA ;
	return "Quotas not supported (for example: quotas are not supported on NFS servers)\n";
    }
    my($bc,$bs,$bh,$bt, $ic,$is,$ih,$it)=Quota::query($dev,$pwent[2],0); 

    unless(defined ($bc) and ($bc > 0 or $bc eq "0")) {
    $cerr += $E_READQINFO and $cerr += { err => (Quota::strerr()) };
	return "Errors while reading quota information. ".(Quota::strerr())."\n";
    }
    ($usedmegs) = &formatquota($bc);
    unless(defined ($bh) and ($bh > 0 or $bh eq "0")) {
    $cerr += $E_READQINFO and $cerr += { err => (Quota::strerr()) };
	return "Errors while reading quota information. ".(Quota::strerr())."\n";
    }
    ($allocatedmegs) = &formatquota($bh);
    return [$usedmegs,$allocatedmegs];

}

#
# This function tells if user and group qoutas are enabled for the
# virtual hosting area. 1:1 means all OK 0:0 means no quota.
#
sub QueryQuota {
    my($usrquota,$grpquota);
    my($dev)=Quota::getqcarg($VHConst::VIRTUALFS);
    if (not defined $dev or not $dev) {
	return "0:0";
    }
    my($bc,$bs,$bh,$bt, $ic,$is,$ih,$it)=Quota::query($dev,0,0); 
    if (defined ($bc) and ($bc > 0 or $bc eq "0")) {
        $usrquota = "1";
    } else {
        $usrquota = "0";
    }
    ($bc,$bs,$bh,$bt, $ic,$is,$ih,$it)=Quota::query($dev,0,1); 
    if (defined ($bc) and ($bc > 0 or $bc eq "0")) {
        $grpquota = "1";
    } else {
        $grpquota = "0";
    }
    return "$usrquota:$grpquota\n";
}


#
# Turns an integer (in kilobytes) into a nice textual representation in megs
#
sub formatquota {
    my ($k)=@_;
    return (int(($k+51.2)/102.4))/10;
}

return 1;
