import xml
import copy
import types

import ensimapplpath
from vh3 import virthost
import dashboard
import DynamicImport
import os

"""
<applinfo>
  <login_name>admin</login_name>
  <http_host>asdf.com:19638</http_host>
  <version>3.5.0-5</version>
  <uptime>503108308</uptime>
  <hostname>asdf.com</hostname>
  <ip_addr>10.0.1.1</ip_addr>
</applinfo>
<ipinfo>
  <domain_count>5</domain_count>
</ipinfo>
<diskquota>
  <sites_partition>
    <disk_used>100</disk_used>
    <disk_avail>1000</disk_avail>
  </sites_partition>
</diskquota>
"""

def siteconf_dict_to_xml(in_dict, doc, add_to_elem):
    dict = copy.deepcopy(in_dict)
    for i in dict.keys():
        elem = doc.createElement(i)
        if dict[i].has_key('version'):
            del dict[i]['version']
        dict_to_xml(dict[i], doc, elem)
        add_to_elem.appendChild(elem)

def list_to_xml(in_list, doc, add_to_elem, tagname):
    for i in in_list:
        elem = doc.createElement(tagname)
        elem.appendChild(doc.createTextNode(str(i)))
        add_to_elem.appendChild(elem)
        
def dict_to_xml(dict, doc, add_to_elem):
    """ strings: normal behaviour
        lists: does a <item>item1</item><item>item2</item> conv """
    for i in dict.keys():
        elem = doc.createElement(i)
        if type(dict[i]) is types.ListType:
            list_to_xml(dict[i], doc, elem, 'item')
        else:
            elem.appendChild(doc.createTextNode(str(dict[i])))
        add_to_elem.appendChild(elem)

class XMLDashboard_1:

    custom_xml = '/etc/appliance/customization/custom_xmldashboard'
    def __init__(self):
        self.custom_xml_mod = None
        if os.path.exists('%s.py' % self.custom_xml):
            self.custom_xml_mod = DynamicImport.loadModule(os.path.basename(self.custom_xml),
                                                      path = \
                                                      os.path.dirname(self.custom_xml))
        
    def toxml(self, doc):
        return doc.toxml()

    def add_custom_xml(self, func_name, doc, add_to_elem):
        if self.custom_xml_mod and hasattr(self.custom_xml_mod, func_name):
            customElem = doc.createElement('custom')
            getattr(self.custom_xml_mod, func_name)(doc, customElem)
            add_to_elem.appendChild(customElem)
        
    def appliance_xml(self):
        aaconf = dashboard.get_appliance_conf()
        doc = xml.dom.getDOMImplementation('minidom').createDocument(None,
                                                                     'appliance',
                                                                     None)
        rootElem = doc.documentElement

        propElem = doc.createElement('properties')
        dict_to_xml(dashboard.get_appliance_conf(), doc, propElem)
        rootElem.appendChild(propElem)

        usageElem = doc.createElement('statistics')
        dict_to_xml(dashboard.get_appliance_usage(), doc, usageElem)
        rootElem.appendChild(usageElem)

        self.add_custom_xml('appliance_custom_xml', doc, rootElem)
            
        return self.toxml(doc)

    def reseller_xml(self, reseller_username):
        raconf = dashboard.get_reseller_conf(reseller_username)
        doc = xml.dom.getDOMImplementation('minidom').createDocument(None,
                                                                     'reseller',
                                                                     None)
        rootElem = doc.documentElement

        propElem = doc.createElement('properties')
        dict_to_xml(dashboard.get_reseller_conf(reseller_username),
                    doc, propElem)
        rootElem.appendChild(propElem)

        usageElem = doc.createElement('statistics')
        dict_to_xml(dashboard.get_reseller_usage(reseller_username),
                    doc, usageElem)
        rootElem.appendChild(usageElem)

        self.add_custom_xml('reseller_custom_xml', doc, rootElem)

        return self.toxml(doc)

    def site_xml(self, site):
        saconf = dashboard.get_site_conf(site)
        doc = xml.dom.getDOMImplementation('minidom').createDocument(None,
                                                                     'site',
                                                                     None)
        rootElem = doc.documentElement

        propElem = doc.createElement('properties')
        siteconf_dict_to_xml(dashboard.get_site_conf(site), doc, propElem)
        rootElem.appendChild(propElem)

        usageElem = doc.createElement('statistics')
        dict_to_xml(dashboard.get_site_usage(site), doc, usageElem)
        rootElem.appendChild(usageElem)
        
        self.add_custom_xml('site_custom_xml', doc, rootElem)

        return self.toxml(doc)

    def user_xml(self, site, user):
        uaconf = dashboard.get_user_conf(site, user)
        doc = xml.dom.getDOMImplementation('minidom').createDocument(None,
                                                                     'user',
                                                                     None)
        rootElem = doc.documentElement

        propElem = doc.createElement('properties')
        dict_to_xml(dashboard.get_user_conf(site,user), doc, propElem)
        rootElem.appendChild(propElem)

        usageElem = doc.createElement('statistics')
        dict_to_xml(dashboard.get_user_usage(site,user), doc, usageElem)
        rootElem.appendChild(usageElem)

        self.add_custom_xml('user_custom_xml', doc, rootElem)

        return self.toxml(doc)

class PrettyXMLDashboard(XMLDashboard_1):
    def toxml(self, doc):
        return doc.toprettyxml()

    def test_xml(self):
        doc = xml.dom.getDOMImplementation('minidom').createDocument(None,
                                                                     'user',
                                                                     None)
        rootElem = doc.documentElement
        dict_to_xml({'asdf': ['1','2']}, doc, rootElem)
        return doc.toxml()
    
def create_dashboard(REQUEST = None):
    if REQUEST:
        return XMLDashboard_1()
    else:
        return PrettyXMLDashboard()
    
if __name__ == '__main__':
    #siteconf,errs = virthost.get_configs_from_site('site1')
    #update_site_usage(copy.copy(siteconf))
    #print user_xml('site1', 'xxx')
    #print create_dashboard(None).test_xml()
    print create_dashboard(None).user_xml('site1','cleo')
